#!/usr/bin/env python
# -*- coding: utf-8 -*-

import logging
logger = logging.getLogger(__name__)
logger.debug("%s loaded", __name__)

import os

import ConfigParser
import doorpi

from backward_compatibility import BACKWARD_COMPATIBILITY_KEYS

class ConfigObject():

    __sections = {}
    _config_file = None

    @property
    def all(self): return self.__sections

    @property
    def config_file(self): return self._config_file

    def __init__(self, config, config_file = None):
        logger.debug("__init__")
        self.get_from_config(config)
        self._config_file = config_file

    def __del__(self):
        return self.destroy()

    def destroy(self):
        logger.debug("__del__")
        #DoorPi().event_handler.unregister_source(__name__, True)
        return True

    @staticmethod
    def find_config(configfile = None):
        try:
            open(configfile.name, 'r').close()
            return configfile.name
        except AttributeError: pass
        except IOError: pass

        default_files = [
            str(configfile),
            doorpi.DoorPi().parse_string(str(configfile)),
            os.path.join('!BASEPATH!', 'conf', 'doorpi.ini'),
            os.path.join('!BASEPATH!', 'conf', 'doorpi.cfg')
        ]

        for possible_default_file in default_files:
            try:
                possible_default_file = doorpi.DoorPi().parse_string(possible_default_file)
                open(possible_default_file, 'r').close()
                return possible_default_file
            except: pass

        return None

    @staticmethod
    def load_config(configfile, search_for_defaults = True):
        config = ConfigParser.ConfigParser(allow_no_value = True)
        if search_for_defaults: configfile_name = ConfigObject.find_config(configfile)
        else: configfile_name = configfile

        if not configfile_name:
            logger.error('No valid configfile found - use parameter --configfile "filename" to specify one')
            return ConfigObject(config)

        logger.info("use configfile: %s", configfile_name)

        config.read(configfile_name)
        return ConfigObject(config, configfile_name)

    def save_config(self, configfile = ''):
        if not configfile: configfile = self.config_file
        if not configfile: configfile = self.find_config(configfile)
        if not configfile: configfile = doorpi.DoorPi().parse_string(os.path.join('!BASEPATH!', 'conf', 'doorpi.ini'))

        #if not configfile: return False
        logger.debug("write configfile: %s", configfile)
        try:
            if not os.path.exists(os.path.dirname(configfile)):
                logger.info('Path %s does not exist - creating it now', os.path.dirname(configfile))
                os.makedirs(os.path.dirname(configfile))
            cfgfile = open(configfile,'w')
            config = ConfigParser.ConfigParser(allow_no_value = True)
            for section in sorted(self.__sections.keys()):
                config.add_section(section)
                for key in sorted(self.__sections[section].keys()):
                    config.set(section, key, self.__sections[section][key])
            config.write(cfgfile)
            cfgfile.close()
            logger.info("write configfile was success: %s", configfile)
            return True
        except Exception as exp:
            logger.exception(exp)
            return False

    def get_string_parsed(self, section, key, default = '', log = True):
        raw_string = self.get_string(section, key, default, log)
        parsed_string = doorpi.DoorPi().parse_string(raw_string)
        logger.debug('parse string "%s" to "%s"', raw_string, parsed_string)
        return parsed_string

    def set_value(self, section, key, value, log = True, password = False):
        if section not in self.__sections.keys():
            self.__sections[section] = {}

        password_friendly_value = "*******" if key is 'password' or password else value

        if key not in self.__sections[section].keys():
            if log: logger.debug("create new key %s in section %s with value '%s'",
                                 key, section, password_friendly_value)
        else:
            if log: logger.debug("overwrite key %s in section %s from '%s' to '%s'",
                                 key, section, self.__sections[section][key], password_friendly_value)

        self.__sections[section][key] = value
        return True

    def rename_key(self, section, old_key, new_key, default = '', log = True):
        if log: logging.debug('rename key from %s to %s in section %s', old_key, new_key, section)
        old_value = self.get_string(section, old_key, default, log, store_if_not_exists = False)
        self.set_value(
            section = section,
            key = new_key,
            value = old_value,
            log = log
        )
        self.delete_key(section, old_key, log = log)

    def delete_section(self, section, delete_empty_only = True, log = True):
        if section in self.__sections and len(self.__sections[section]) > 0 and delete_empty_only:
            logger.warning("could not delete section %s, because it's not empty.", section)
            return False

        try:
            if len(self.__sections[section]) > 0 and delete_empty_only:
                raise KeyError('section is not empty')

            self.__sections.pop(section)
            return True
        except KeyError as exp:
            if log: logger.warning('delete section %s failed: %s', section, exp)
            return False

    def delete_key(self, section, key, log = True):
        try:
            if log: logger.info('delete key %s from section %s', key, section)
            self.__sections[section].pop(key)
            self.delete_section(section, log = log)

            return True
        except KeyError as exp:
            if log: logger.warning('delete key %s from section %s failed: %s', key, section, exp)
        return False

    def get_string(self, section, key, default = '', log = True, password = False, store_if_not_exists = True):
        value = None
        try:
            old_section, old_key = BACKWARD_COMPATIBILITY_KEYS[section][key]
            value = self.__sections[old_section][old_key]
            self.delete_key(old_section, old_key, False)
            self.__sections[section][key] = value
            logger.warning('found %s - %s in BACKWARD_COMPATIBILITY_KEYS with %s - %s', section, key, old_section, old_key)
        except KeyError:
            try:
                value = self.__sections[section][key]
            except KeyError:
                pass
        
        if value is None:
            #logger.trace('no value found - use default')
            value = default
            if store_if_not_exists: self.set_value(section, key, default, log, password)

        if key.endswith('password') or password:
            if log: logger.trace("get_string for key %s in section %s (default: %s) returns %s", key, section, default, '*******')
        else:
            if log: logger.trace("get_string for key %s in section %s (default: %s) returns %s", key, section, default, value)
        return value

    def get_float(self, section, key, default = -1, log = True, store_if_not_exists = True):
        value = self.get_string(section, key, str(default), log = False, store_if_not_exists = store_if_not_exists)
        if value is not '': value = float(value)
        else: value = default
        if log: logger.trace("get_integer for key %s in section %s (default: %s) returns %s", key, section, default, value)
        return value

    def get_integer(self, section, key, default = -1, log = True, store_if_not_exists = True):
        value = self.get(section, key, str(default), log = False, store_if_not_exists = store_if_not_exists)
        if value is not '': value = int(value)
        else: value = default
        if log: logger.trace("get_integer for key %s in section %s (default: %s) returns %s", key, section, default, value)
        return value

    def get_boolean(self, section, key, default = False, log = True, store_if_not_exists = True):
        value = self.get(section, key, str(default), log = False, store_if_not_exists = store_if_not_exists)
        value = value.lower() in ['true', 'yes', 'ja', '1']
        if log: logger.trace("get_boolean for key %s in section %s (default: %s) returns %s", key, section, default, value)
        return value

    def get_list(self, section, key, default = [], separator = ',', log = True, store_if_not_exists = True):
        value = self.get(section, key, str(default), log = False, store_if_not_exists = store_if_not_exists)
        if value is not '': value = value.split(separator)
        else: value = default
        if log: logger.trace("get_list for key %s in section %s (default: %s) returns %s", key, section, default, value)
        return value

    def get_sections(self, filter = '', log = True):
        return_list = []
        for section in self.__sections:
            if filter in section: return_list.append(section)
        if log: logger.trace("get_sections returns %s", return_list)
        return return_list

    def get_keys(self, section, filter = '', log = True):
        return_list = []
        if section not in self.__sections:
            logging.warning("section %s not found in configfile", section)
        else:
            for key in self.__sections[section]:
                if filter in key: return_list.append(key)
        if log: logger.trace("get_keys for section %s returns %s", section, return_list)
        return return_list

    def get_from_config(self, config, log = True):
        if log: logger.trace("get_from_config")
        for section in config.sections():
            self.__sections[section] = {}
            for key, value in config.items(section):
                if key.startswith(';') or key.startswith('#'): continue
                self.__sections[section][str(key)] = str(value)

    get = get_string
    get_bool = get_boolean
    get_int = get_integer



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import logging
logger = logging.getLogger(__name__)
logger.debug("%s loaded", __name__)

import os

import doorpi
from doorpi.media.CreateDialTone import generate_dial_tone
from doorpi.sipphone.AbstractBaseClass import PlayerAbstractBaseClass, SIPPHONE_SECTION

class LinphonePlayer(PlayerAbstractBaseClass):

    __player_id = None
    __slot_id = None

    __player_filename = ''

    @property
    def player_filename(self): return self.__player_filename

    def __init__(self):
        self.__player_filename = doorpi.DoorPi().config.get_string_parsed(SIPPHONE_SECTION, 'dialtone',
                                                                          '!BASEPATH!/media/ShortDialTone.wav')
        if self.__player_filename is '':
            logger.debug('no player found in config at section DoorPi and key dialtone')
            return

        doorpi.DoorPi().event_handler.register_action('OnSipPhoneDestroy', self.destroy)

        self.__player_filename = doorpi.DoorPi().parse_string(self.__player_filename)
        if not os.path.exists(os.path.dirname(self.__player_filename)):
            logger.info('Path %s does not exist - creating it now', os.path.dirname(self.__player_filename))
            os.makedirs(os.path.dirname(self.__player_filename))
        dialtone_renew_every_start = doorpi.DoorPi().config.get_bool(SIPPHONE_SECTION, 'dialtone_renew_every_start', False)
        if not os.path.isfile(self.__player_filename) or dialtone_renew_every_start:
            logger.info('DialTone %s does not exist - creating it now', self.__player_filename)
            dialtone_volume = doorpi.DoorPi().config.get_int(SIPPHONE_SECTION, 'dialtone_volume', 35)
            generate_dial_tone(self.__player_filename, dialtone_volume)
        doorpi.DoorPi().event_handler.register_event('OnPlayerStarted', __name__)
        doorpi.DoorPi().event_handler.register_event('OnPlayerStopped', __name__)
        doorpi.DoorPi().event_handler.register_event('OnPlayerCreated', __name__)

        doorpi.DoorPi().event_handler.register_action('OnSipPhoneMakeCall', self.start)
        doorpi.DoorPi().event_handler.register_action('OnCallStateConnect', self.stop)
        doorpi.DoorPi().event_handler.register_action('OnCallStateDisconnect', self.stop)

        doorpi.DoorPi().event_handler('OnPlayerCreated', __name__)

    def start(self): doorpi.DoorPi().event_handler('OnPlayerStarted', __name__)
    def stop(self):  doorpi.DoorPi().event_handler('OnPlayerStopped', __name__)
    def destroy(self):
        self.stop()
        doorpi.DoorPi().event_handler.unregister_source(__name__, True)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import logging
logger = logging.getLogger(__name__)
logger.debug("%s loaded", __name__)

import subprocess
from doorpi.action.base import SingleAction
import doorpi

def fire_command(command):
    return subprocess.Popen(
        command,
        shell = True,
        stdout = subprocess.PIPE
    ).stdout.read()

def get(parameters):
    parsed_parameters = doorpi.DoorPi().parse_string(parameters)
    return OsExecuteAction(fire_command, command = parsed_parameters)

class OsExecuteAction(SingleAction):
    pass


#!/usr/bin/env python
# -*- coding: utf-8 -*-

import logging
logger = logging.getLogger(__name__)
logger.debug("%s loaded", __name__)

from doorpi.action.base import SingleAction
import doorpi

def write_status_watchdog(watchdog_path, timeout):
    timeout = int(timeout)

    try:
        watchdog = open(watchdog_path, "w+")
    except:
        logger.warning("while action write_status_watchdog - error opening watchdog file")
        return False

    try:
        watchdog.write('\n')
        watchdog.flush()
    finally:
        watchdog.close()

    return True

def get(parameters):
    parameter_list = parameters.split(',')
    if len(parameter_list) is not 1 and len(parameter_list) is not 2: return None

    watchdog = parameter_list[0]
    timeout = 5

    if len(parameter_list) is 2:
        timeout = int(parameter_list[1])

    return SleepAction(write_status_watchdog, watchdog, timeout)

class SleepAction(SingleAction):
    pass


#!/usr/bin/env python
# -*- coding: utf-8 -*-

import logging
logger = logging.getLogger(__name__)
logger.debug("%s loaded", __name__)

def get(*args, **kwargs):
    try:
        if len(kwargs['name']) == 0: kwargs['name'] = ['']
        if len(kwargs['value']) == 0: kwargs['value'] = ['']
        return_dict = {}
        for section_request in kwargs['name']:
            for section in kwargs['DoorPiObject'].config.get_sections(section_request):
                return_dict[section] = {}
                for value_request in kwargs['value']:
                    for key in kwargs['DoorPiObject'].config.get_keys(section, value_request):
                        return_dict[section][key] = kwargs['DoorPiObject'].config.get(section, key)

        for section in return_dict.keys():
            if len(return_dict[section]) == 0: del return_dict[section]

        return return_dict
    except Exception as exp:
        logger.exception(exp)
        return {'Error': 'could not create '+str(__name__)+' object - '+str(exp)}

def is_active(doorpi_object):
    return True if doorpi_object.config else False



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import logging
logger = logging.getLogger(__name__)
logger.debug("%s loaded", __name__)

import time # session timestamp

from doorpi.action.base import SingleAction
import doorpi

CONF_AREA_PREFIX = 'AREA_'

class SessionHandler:

    _Sessions = {}

    @property
    def config(self): return doorpi.DoorPi().config

    @property
    def session_ids(self): return self._Sessions.keys()

    @property
    def sessions(self): return self._Sessions

    def __init__(self):
        doorpi.DoorPi().event_handler.register_event('WebServerCreateNewSession', __name__)
        doorpi.DoorPi().event_handler.register_event('WebServerAuthUnknownUser', __name__)
        doorpi.DoorPi().event_handler.register_event('WebServerAuthWrongPassword', __name__)

    def destroy(self):
        doorpi.DoorPi().event_handler.unregister_source(__name__, True)

    __del__ = destroy

    def get_session(self, session_id):
        if session_id in self._Sessions:
            logger.trace('session %s found: %s', session_id, self._Sessions[session_id])
            return self._Sessions[session_id]
        else:
            logger.trace('no session with session id %s found', session_id)
            return None

    __call__ = get_session

    def exists_session(self, session_id):
        return session_id in self._Sessions

    def build_security_object(self, username, password, remote_client = ''):
        if not len(self.config.get_keys('User')):
            self.config.set_value(section = 'User', key = 'door', value = 'pi', password = True)
            self.config.set_value(section = 'Group', key = 'administrator', value = 'door')
            self.config.set_value(section = 'WritePermission', key = 'administrator', value = 'installer')
            self.config.set_value(section = 'AREA_installer', key = '.*', value = '')

        groups_with_write_permissions = self.config.get_keys('WritePermission')
        groups_with_read_permissions = self.config.get_keys('ReadPermission')
        groups = self.config.get_keys('Group')
        users = self.config.get_keys('User')

        if not username in users:
            doorpi.DoorPi().event_handler('WebServerAuthUnknownUser', __name__, {
                'username': username,
                'remote_client': remote_client
            })
            return None

        real_password = self.config.get('User', username, password = True)
        if real_password != password:
            doorpi.DoorPi().event_handler('WebServerAuthWrongPassword', __name__, {
                'username': username,
                'password': password,
                'remote_client': remote_client
            })
            return None

        web_session = dict(
            username = username,
            remote_client = remote_client,
            session_starttime = time.time(),
            readpermissions = [],
            writepermissions = [],
            groups = []
        )

        for group in groups:
            users_in_group = self.config.get_list('Group', group)
            if username in users_in_group: web_session['groups'].append(group)

        for group in groups_with_read_permissions:
            if group in web_session['groups']:
                modules = self.config.get_list('ReadPermission', group)
                for modul in modules:
                    web_session['readpermissions'].extend(
                        self.config.get_keys(CONF_AREA_PREFIX+modul)
                    )

        for group in groups_with_write_permissions:
            if group in web_session['groups']:
                modules = self.config.get_list('WritePermission', group)
                for modul in modules:
                    web_session['writepermissions'].extend(
                        self.config.get_keys(CONF_AREA_PREFIX+modul)
                    )
                    web_session['readpermissions'].extend(
                        self.config.get_keys(CONF_AREA_PREFIX+modul)
                    )

        web_session['readpermissions'] = list(set(web_session['readpermissions']))
        web_session['readpermissions'].sort()
        web_session['writepermissions'] = list(set(web_session['writepermissions']))
        web_session['writepermissions'].sort()

        doorpi.DoorPi().event_handler('WebServerCreateNewSession', __name__, {
            'session':  web_session
        })

        self._Sessions[web_session['username']] = web_session
        return web_session



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import logging
logger = logging.getLogger(__name__)
logger.debug("%s loaded", __name__)

import threading
import datetime
import time
import os
import pjsua as pj
from doorpi import DoorPi
from SipPhoneCallCallBack import SipPhoneCallCallBack as CallCallback

class SipPhoneAccountCallBack(pj.AccountCallback):

    sem = None

    def __init__(self, account = None):
        logger.debug("__init__")
        pj.AccountCallback.__init__(self, account)
        DoorPi().event_handler.register_event('BeforeCallIncoming', __name__)
        DoorPi().event_handler.register_event('OnCallReconnect', __name__)
        DoorPi().event_handler.register_event('AfterCallReconnect', __name__)
        DoorPi().event_handler.register_event('OnCallBusy', __name__)
        DoorPi().event_handler.register_event('AfterCallBusy', __name__)
        DoorPi().event_handler.register_event('OnCallIncoming', __name__)
        DoorPi().event_handler.register_event('AfterCallIncoming', __name__)
        DoorPi().event_handler.register_event('OnCallReject', __name__)
        DoorPi().event_handler.register_event('AfterCallReject', __name__)
        #DoorPi().event_handler.register_event('AfterAccountRegState', __name__)

    def __del__(self):
        self.destroy()

    def destroy(self):
        logger.debug("destroy")
        DoorPi().event_handler.unregister_source(__name__, True)

    def wait(self):
        self.sem = threading.Semaphore(0)
        self.sem.acquire()

    def on_reg_state(self):
        if self.sem:
            if self.account.info().reg_status >= 200:
                self.sem.release()

        #DoorPi().event_handler('AfterAccountRegState', __name__)
        #logger.debug(self.account.info.reg_status)

    def answer_call(self, call):
        DoorPi().sipphone.current_callcallback = CallCallback()
        call.set_callback(DoorPi().sipphone.current_callcallback)
        DoorPi().sipphone.current_call = call
        DoorPi().sipphone.current_call.answer(code = 200)

    def on_incoming_call(self, call):
        # SIP-Status-Codes: http://de.wikipedia.org/wiki/SIP-Status-Codes
        # 200 = OK
        # 401 = Unauthorized
        # 403 = Forbidden
        # 486 = Busy Here
        # 494 = Security Agreement Required
        logger.debug("on_incoming_call")
        logger.info("Incoming call from %s", str(call.info().remote_uri))
        DoorPi().event_handler('BeforeCallIncoming', __name__)

        call.answer(180)

        if DoorPi().sipphone.current_call is not None and DoorPi().sipphone.current_call.is_valid():
            logger.debug("Incoming call while another call is active")
            logger.debug("- incoming.remote_uri: %s", call.info().remote_uri)
            logger.debug("- current.remote_uri : %s", DoorPi().sipphone.current_call.info().remote_uri)

            if call.info().remote_uri == DoorPi().sipphone.current_call.info().remote_uri:
                logger.info("Current call is incoming call - quitting current and connecting to incoming. Maybe connection reset?")
                DoorPi().event_handler('OnCallReconnect', __name__, {'remote_uri': call.info().remote_uri})
                DoorPi().current_call.hangup()
                self.answer_call(call)
                DoorPi().event_handler('AfterCallReconnect', __name__)
                return
            else:
                logger.info("Incoming and current call are different - sending busy signal to incoming call")
                DoorPi().event_handler('OnCallBusy', __name__, {'remote_uri': call.info().remote_uri})
                call.answer(code = 494, reason = "Security Agreement Required")
                DoorPi().event_handler('AfterCallBusy', __name__)
                return

        if DoorPi().sipphone.is_admin_number(call.info().remote_uri):
            logger.debug("Incoming call from trusted admin number %s -> autoanswer", call.info().remote_uri)
            DoorPi().event_handler('OnCallIncoming', __name__, {'remote_uri': call.info().remote_uri})
            self.answer_call(call)
            DoorPi().event_handler('AfterCallIncoming', __name__)
            return
        else:
            logger.debug("Incoming call ist not from a trusted admin number %s -> sending busy signal", call.info().remote_uri)
            DoorPi().event_handler('OnCallReject', __name__, {'remote_uri': call.info().remote_uri})
            call.answer(code = 494, reason = "Security Agreement Required")
            DoorPi().event_handler('AfterCallReject', __name__)
            return


#!/usr/bin/env python
# -*- coding: utf-8 -*-

import logging
logger = logging.getLogger(__name__)
logger.debug("%s loaded", __name__)

from datetime import datetime

def get(*args, **kwargs):
    return str(datetime.now())

def is_active(doorpi_object):
    return True



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import logging
logger = logging.getLogger(__name__)
logger.debug("%s loaded", __name__)

import datetime

from AbstractBaseClass import SipphoneAbstractBaseClass, SIPPHONE_SECTION
import linphone as lin

from doorpi import DoorPi
from doorpi.sipphone.linphone_lib.CallBacks import LinphoneCallbacks
from doorpi.sipphone.linphone_lib.Player import LinphonePlayer
from doorpi.sipphone.linphone_lib.Recorder import LinphoneRecorder
from doorpi.media.CreateDialTone import generate_dial_tone

conf = DoorPi().config

def log_handler(level, msg):
    if "pylinphone_Core_instance_method_iterate" in msg: return
    if "pylinphone_Core_get_current_call" in msg: return
    if "pylinphone_Call_from_native_ptr" in msg: return
    if ": keep alive sent to [" in msg: return
    method = getattr(logger, level)
    method(msg)

if logger.getEffectiveLevel() <= 5: lin.set_log_handler(log_handler)

def get(*args, **kwargs): return LinPhone(*args, **kwargs)
class LinPhone(SipphoneAbstractBaseClass):

    @property
    def name(self): return 'linphone wrapper'

    @property
    def lib(self): return self.__Lib
    @property
    def core(self): return self.__Lib

    @property
    def recorder(self): return self.__recorder
    __recorder = None

    @property
    def player(self): return self.__player
    __player = None

    @property
    def current_call(self): return self.core.current_call

    @property
    def video_devices(self):
        try:
            all_devices = []
            for video_device in self.core.video_devices:
                all_devices.append({
                  'name':       video_device
                })
            return all_devices
        except Exception:
            return []

    @property
    def sound_devices(self):
        try:
            all_devices = []
            for sound_device in self.core.sound_devices:
                all_devices.append({
                  'name':       sound_device,
                  'capture':    self.core.sound_device_can_capture(sound_device),
                  'record':     self.core.sound_device_can_playback(sound_device)
                })
            return all_devices
        except Exception as exp:
            logger.exception(exp)
            return []

    def _create_payload_enum(self, payloads):

        try:
            all_codecs = []
            for codec in payloads:
                all_codecs.append({
                    'name':         codec.mime_type,
                    'channels':     codec.channels,
                    'bitrate':      codec.normal_bitrate,
                    'enable':       self.core.payload_type_enabled(codec)
                })
            return all_codecs
        except Exception as exp:
            logger.exception(exp)
            return []

    @property
    def video_codecs(self):
        return self._create_payload_enum(self.core.video_codecs)

    @property
    def sound_codecs(self):
        return self._create_payload_enum(self.core.audio_codecs)

    @property
    def current_call_duration(self):
        if not self.current_call: return 0
        diff_start_and_now = datetime.datetime.utcnow() - self.__current_call_start_datetime
        return diff_start_and_now.total_seconds()

    @property
    def current_call_dump(self):
        try:
            return {
                'direction':        'incoming' if self.current_call.dir == 0 else 'outgoing',
                'remote_uri':       self.current_call.remote_address_as_string,
                'total_time':       self.current_call_duration,
                'level_incoming':   self.current_call.record_volume,
                'level_outgoing':   self.current_call.play_volume,
                'camera':           self.current_call.camera_enabled
            }
        except Exception:
            return {}

    #TODO: Datetime from linphone CallLog.start_date is more then 30 sec different to python datetime.utcnow()?
    __current_call_start_datetime = datetime.datetime.utcnow()

    @property
    def base_config(self):
        params = self.core.create_call_params(None)
        params.record_file = self.recorder.parsed_record_filename
        params.video_enabled = True
        return params

    def reset_call_start_datetime(self):
        self.__current_call_start_datetime = datetime.datetime.utcnow()
        logger.debug('reset current call start datetime to %s', self.__current_call_start_datetime)
        return self.__current_call_start_datetime

    def __init__(self, whitelist = list(), *args, **kwargs):
        logger.debug("__init__")

        DoorPi().event_handler.register_action('OnShutdown', self.destroy)

        DoorPi().event_handler.register_event('OnSipPhoneCreate', __name__)
        DoorPi().event_handler.register_event('OnSipPhoneStart', __name__)
        DoorPi().event_handler.register_event('OnSipPhoneDestroy', __name__)

        DoorPi().event_handler.register_event('OnSipPhoneRecorderCreate', __name__)
        DoorPi().event_handler.register_event('OnSipPhoneRecorderDestroy', __name__)

        DoorPi().event_handler.register_event('BeforeSipPhoneMakeCall', __name__)
        DoorPi().event_handler.register_event('OnSipPhoneMakeCall', __name__)
        DoorPi().event_handler.register_event('OnSipPhoneMakeCallFailed', __name__)
        DoorPi().event_handler.register_event('AfterSipPhoneMakeCall', __name__)
        
        DoorPi().event_handler.register_event('OnSipPhoneCallTimeoutNoResponse', __name__)
        DoorPi().event_handler.register_event('OnSipPhoneCallTimeoutMaxCalltime', __name__)

        DoorPi().event_handler.register_event('OnPlayerCreated', __name__)

        #http://pythonhosted.org/linphone/api_reference.html#linphone.Core.new
        self.callback = LinphoneCallbacks()
        config_path = None
        factory_config_path = None
        self.__Lib = lin.Core.new(
            self.callback.used_callbacks,
            config_path,
            factory_config_path
        )
        self.core.primary_contact = '%s <sip:doorpi@127.0.0.1>'%conf.get(SIPPHONE_SECTION, "identity", 'DoorPi')

    def start(self):
        DoorPi().event_handler('OnSipPhoneCreate', __name__)
        self.core.max_calls = conf.get_int(SIPPHONE_SECTION, 'ua.max_calls', 2)
        self.core.echo_cancellation_enabled = conf.get_bool(SIPPHONE_SECTION, 'echo_cancellation_enabled', False)
        
        # set local listen ports, default: random
        self.core.sip_transports = lin.SipTransports(conf.get_int(SIPPHONE_SECTION, 'local_port', 5060), conf.get_int(SIPPHONE_SECTION, 'local_port', 5060), -1, -1)

        self.core.video_display_enabled = conf.get_bool(SIPPHONE_SECTION, 'video_display_enabled', False)
        self.core.stun_server = conf.get(SIPPHONE_SECTION, 'stun_server', '')
        firewall_policy = conf.get(SIPPHONE_SECTION, 'FirewallPolicy', 'PolicyNoFirewall')
        if firewall_policy == "PolicyNoFirewall": self.core.firewall_policy = lin.FirewallPolicy.PolicyNoFirewall
        elif firewall_policy == "PolicyUseNatAddress": self.core.firewall_policy = lin.FirewallPolicy.PolicyUseNatAddress
        elif firewall_policy == "PolicyUseStun": self.core.firewall_policy = lin.FirewallPolicy.PolicyUseStun
        elif firewall_policy == "PolicyUseIce": self.core.firewall_policy = lin.FirewallPolicy.PolicyUseIce
        elif firewall_policy == "PolicyUseUpnp": self.core.firewall_policy = lin.FirewallPolicy.PolicyUseUpnp
        else: self.core.firewall_policy = lin.FirewallPolicy.PolicyNoFirewall

        #http://pythonhosted.org/linphone/api_reference.html#linphone.Core.in_call_timeout
        #After this timeout period, the call is automatically hangup.
        self.core.in_call_timeout = conf.get_int(SIPPHONE_SECTION, 'max_call_time', 120)
        #http://pythonhosted.org/linphone/api_reference.html#linphone.Core.inc_timeout
        #If an incoming call isn’t answered for this timeout period, it is automatically declined.
        self.core.inc_timeout = conf.get_int(SIPPHONE_SECTION, 'call_timeout', 15)

        self.__player = LinphonePlayer()
        self.core.ringback = self.player.player_filename
        self.__recorder = LinphoneRecorder()

        if len(self.core.sound_devices) == 0:
            logger.warning('no audio devices available')
        else:
            self.core.capture_device = conf.get(SIPPHONE_SECTION, 'capture_device', self.core.capture_device)
            self.core.playback_device = conf.get(SIPPHONE_SECTION, 'playback_device', self.core.playback_device)
            logger.info("found %s possible sounddevices:", len(self.core.sound_devices))
            logger.debug("|rec|play| name")
            logger.debug("------------------------------------")
            for sound_device in self.core.sound_devices:
                logger.debug("| %s | %s  | %s",
                    'X' if self.core.sound_device_can_capture(sound_device) else 'O',
                    'X' if self.core.sound_device_can_playback(sound_device) else 'O',
                    sound_device
                )
            logger.debug("------------------------------------")
            logger.debug("using capture_device: %s", self.core.capture_device)
            logger.debug("using playback_device: %s", self.core.playback_device)

        # Only enable PCMU and PCMA audio codecs by default
        config_audio_codecs = conf.get_list(SIPPHONE_SECTION, 'audio_codecs', 'PCMA,PCMU')
        for codec in self.core.audio_codecs:
            if codec.mime_type in config_audio_codecs:
                logger.debug('enable audio codec %s', codec.mime_type)
                self.core.enable_payload_type(codec, True)
            else:
                logger.debug('disable audio codec %s', codec.mime_type)
                self.core.enable_payload_type(codec, False)


        if len(self.core.video_devices) == 0:
            self.core.video_capture_enabled = False
            logger.warning('no video devices available')
        else:
            logger.info("found %s possible videodevices:", len(self.core.video_devices))
            logger.debug("| name")
            logger.debug("------------------------------------")
            for video_device in self.core.video_devices:
                logger.debug("| %s ", video_device)
            logger.debug("------------------------------------")
            config_camera = conf.get(SIPPHONE_SECTION, 'video_device', self.core.video_devices[0])
            if config_camera not in self.core.video_devices:
                logger.warning('camera "%s" from config does not exist in possible video devices.', config_camera)
                logger.debug('switching to first possible video device "%s"', self.core.video_devices[0])
                config_camera = self.core.video_devices[0]

            self.core.video_capture_enabled = True
            self.core.video_device = config_camera
            self.core.preferred_video_size_by_name = conf.get(SIPPHONE_SECTION, 'video_size', 'vga')
            logger.debug("using video_device: %s", self.core.video_device)

        # Only enable VP8 video codec
        config_video_codecs = conf.get_list(SIPPHONE_SECTION, 'video_codecs', 'VP8')
        for codec in self.core.video_codecs:
            if codec.mime_type in config_video_codecs and self.core.video_capture_enabled:
                logger.debug('enable video codec %s', codec.mime_type)
                self.core.enable_payload_type(codec, True)
            else:
                logger.debug('disable video codec %s', codec.mime_type)
                self.core.enable_payload_type(codec, False)

        # Configure the SIP account
        server = conf.get(SIPPHONE_SECTION, "sipserver_server")
        username = conf.get(SIPPHONE_SECTION, "sipserver_username")
        password = conf.get(SIPPHONE_SECTION, "sipserver_password", username)
        realm = conf.get(SIPPHONE_SECTION, "sipserver_realm", server)
        if server and username and password:
            logger.info('using DoorPi with SIP-Server')
            proxy_cfg = self.core.create_proxy_config()
            proxy_cfg.identity_address = lin.Address.new("%s <sip:%s@%s>" % (
                    conf.get(SIPPHONE_SECTION, "identity", 'DoorPi'), username, server)
            )
            proxy_cfg.server_addr = "sip:%s"%server
            proxy_cfg.register_enabled = True
            self.core.add_proxy_config(proxy_cfg)
            self.core.default_proxy_config = proxy_cfg
            auth_info = self.core.create_auth_info(username, None, password, None, None, realm)
            self.core.add_auth_info(auth_info)
        else:
            logger.info('using DoorPi without SIP-Server? Okay...')
            proxy_cfg = self.core.create_proxy_config()
            proxy_cfg.register_enabled = False
            self.core.add_proxy_config(proxy_cfg)
            self.core.default_proxy_config = proxy_cfg
            logger.debug('%s',self.core.proxy_config_list)

        logger.debug("start successfully")

    def destroy(self):
        logger.debug("destroy")
        self.core.terminate_all_calls()
        DoorPi().event_handler.fire_event_synchron('OnSipPhoneDestroy', __name__)
        DoorPi().event_handler.unregister_source(__name__, True)
        return

    def self_check(self, *args, **kwargs):
        if not self.core: return

        self.core.iterate()

        if not self.current_call: return

        if self.current_call.state < lin.CallState.Connected:
            if self.current_call_duration > self.core.inc_timeout - 0.5:
                logger.info("call timeout - hangup current call after %s seconds (max. %s)", self.current_call_duration, self.core.inc_timeout)
                self.core.terminate_all_calls()
                DoorPi().event_handler('OnSipPhoneCallTimeoutNoResponse', __name__)
        else:
            if int(self.current_call_duration) > self.core.in_call_timeout - 0.5:
                logger.info("max call time reached - hangup current call after %s seconds (max. %s)", self.current_call_duration, self.core.in_call_timeout)
                self.core.terminate_all_calls()
                DoorPi().event_handler('OnSipPhoneCallTimeoutMaxCalltime', __name__)

    def call(self, number):
        DoorPi().event_handler('BeforeSipPhoneMakeCall', __name__, {'number':number})
        logger.debug("call (%s)",str(number))
        if not self.current_call:
            logger.debug('no current call -> start new call')
            self.reset_call_start_datetime()
            if self.core.invite_with_params(number, self.base_config) is None:
                if DoorPi().event_handler.db.get_event_log_entries_count('OnSipPhoneMakeCallFailed') > 5:
                    logger.error('failed to execute call five times')
                else:
                    DoorPi().event_handler('OnSipPhoneMakeCallFailed', __name__, {'number':number})
                return None
            DoorPi().event_handler('OnSipPhoneMakeCall', __name__, {'number':number})
        elif number in self.current_call.remote_address.as_string_uri_only():
            if self.current_call_duration <= 2:
                logger.debug("same call %s again while call is running since %s seconds? -> skip",
                             self.core.current_call.remote_address.as_string_uri_only(),
                             self.current_call_duration
                )
            else:
                logger.debug("press twice with call duration > 1 second? Want to hangup current call? OK...")
                self.core.terminate_all_calls()
        else:
            logger.debug("new call needed? hangup old first...")
            self.core.terminate_all_calls()
            self.call(number)

        DoorPi().event_handler('AfterSipPhoneMakeCall', __name__, {'number':number})
        return self.current_call

    def is_admin_number(self, remote_uri):
        return self.callback.is_admin_number(remote_uri)

    def hangup(self):
        if self.current_call:
            logger.debug("Received hangup request, cancelling current call")
            self.core.terminate_call(self.current_call)
        else:
            logger.debug("Ignoring hangup request as there is no ongoing call")




#!/usr/bin/env python
# -*- coding: utf-8 -*-

import logging
logger = logging.getLogger(__name__)
logger.debug("%s loaded", __name__)

from doorpi.action.base import SingleAction
import doorpi

from time import sleep

def sipphone_calltimeout(timeout, *callstate_to_check):
    #import pjsua
    try:
        doorpi.DoorPi().sipphone.lib.thread_register('pjsip_handle_events')
        return doorpi.DoorPi().sipphone.call_timeout(timeout, callstate_to_check)
    except:
        return False

def get(parameters):
    parameter_list = parameters.split(',')
    if len(parameter_list) < 1: return None

    timeout = int(parameter_list[0])
    callstate_to_check = parameter_list[1:]

    return SipphoneCallTimeoutAction(sipphone_calltimeout, timeout = timeout)

class SipphoneCallTimeoutAction(SingleAction):
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import logging
import os
logger = logging.getLogger(__name__)
logger.debug("%s loaded", __name__)

SIPPHONE_SECTION = 'SIP-Phone'
DOORPI_SECTION = 'DoorPi'

from doorpi.action.base import SingleAction
import doorpi
import subprocess as sub

conf = doorpi.DoorPi().config

def take_snapshot(size, path, max):
    if not os.path.exists(path):
        logger.info('Path (%s) does not exist - creating it now', path)
        os.makedirs(path)

    lastFile = getLastFilename(path)
    lastNr = 1
    if (len(lastFile) > 0):
        lastNr = int(lastFile[:lastFile.rfind(".jpg")])
        if (lastNr+1 <= max):
            lastNr = lastNr + 1
    else:
        lastNr = 1
    imageFilename = path + str(lastNr) + ".jpg"
    # fswebcam automatically selects the first video device
    command = "fswebcam --top-banner -b --font luxisr:20 -r " + size + " " + imageFilename
    p = sub.Popen(command, shell=True, stdout=sub.PIPE, stderr=sub.PIPE)
    output, errors = p.communicate()
    if (len(errors) > 0):
        logger.error('error creating snapshot - maybe fswebcam is missing')
    else:
        logger.info('snapshot created: %s', imageFilename)
    return


def getLastFilename(path):
    files = [s for s in os.listdir(path)
         if os.path.isfile(os.path.join(path, s))]
    files.sort(key=lambda s: os.path.getmtime(os.path.join(path, s)))
    if (len(files) == 0):
        return ''
    return files[len(files)-1]

def get(parameters):
    conf = doorpi.DoorPi().config
    snapshot_size = conf.get_string(DOORPI_SECTION, 'snapshot_size', '1280x720')
    snapshot_path = conf.get_string_parsed(DOORPI_SECTION, 'snapshot_path', '!BASEPATH!/../DoorPiWeb/snapshots/')
    number_of_snapshots = conf.get_int(DOORPI_SECTION, 'number_of_snapshots', 10)

    if len(conf.get(SIPPHONE_SECTION, 'capture_device', '')) > 0:
        return SnapShotAction(take_snapshot, size = snapshot_size, path = snapshot_path, max = number_of_snapshots)
    logger.warning('can not create snapshot - video disabled')

class SnapShotAction(SingleAction):
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import logging
logger = logging.getLogger(__name__)
logger.debug("%s loaded", __name__)

from time import sleep
import importlib

class SingleAction:
    action_name = None
    single_fire_action = False

    @property
    def name(self):
        return "%s with args %s and kwargs %s" % (
            self.action_name,
            self.__args,
            self.__kwargs
        )

    def __init__(self, callback, *args, **kwargs):
        self.__callback = callback
        self.__args = args
        self.__kwargs = kwargs
        if len(self.__class__.__bases__) is 0:
            self.action_name = str(callback)
        else:
            self.action_name = self.__class__.__name__

    def __str__(self):
        return self.name

    def run(self, silent_mode = False):
        if not silent_mode:
            logger.trace('run %s with args %s and kwargs %s',
                         self.__class__.__name__,
                         self.__args,
                         self.__kwargs
            )
        try:
            if len(self.__args) is not 0 and len(self.__kwargs) is not 0:
                #print "args and kwargs"
                return self.__callback(*self.__args, **self.__kwargs)
            elif len(self.__args) is 0 and len(self.__kwargs) is not 0:
                #print "no args but kwargs"
                return self.__callback(**self.__kwargs)
            elif len(self.__args) is not 0 and len(self.__kwargs) is 0:
                #print "args and no kwargs"
                return self.__callback(*self.__args)
            else:
                #print "no args and no kwargs"
                return self.__callback()
        except TypeError as ex:
            logger.exception(ex)

    @staticmethod
    def from_string(config_string):
        try:
            action_name = config_string.split(':', 1)[0]
            try: parameters = config_string.split(':', 1)[1]
            except: parameters = ""
            return importlib.import_module('doorpi.action.SingleActions.'+action_name).get(
                parameters
            )
        except:
            logger.exception('error while creating SingleAction from config string: %s',config_string)
            return None


#!/usr/bin/env python
# -*- coding: utf-8 -*-

import logging
logger = logging.getLogger(__name__)
logger.debug("%s loaded", __name__)

import smtplib # used by: fire_action_mail
from email.mime.multipart import MIMEMultipart # used by: fire_action_mail
from email.mime.text import MIMEText # used by: fire_action_mail
from email.MIMEBase import MIMEBase # used by: fire_action_mail
from email import Encoders # used by: fire_action_mail
from email.Utils import COMMASPACE # used by: fire_action_mail

from doorpi.action.base import SingleAction
import doorpi
import os
import subprocess as sub

def fire_action_mail(smtp_to, smtp_subject, smtp_text, smtp_snapshot):
    try:
        smtp_host = doorpi.DoorPi().config.get('SMTP', 'server')
        smtp_port = doorpi.DoorPi().config.get_int('SMTP', 'port')
        smtp_user = doorpi.DoorPi().config.get('SMTP', 'username')
        smtp_password = doorpi.DoorPi().config.get('SMTP', 'password')
        smtp_from = doorpi.DoorPi().config.get('SMTP', 'from')

        smtp_use_tls = doorpi.DoorPi().config.get_boolean('SMTP', 'use_tls')
        smtp_need_login = doorpi.DoorPi().config.get_boolean('SMTP', 'need_login')

        smtp_tolist = smtp_to.split()

        server = smtplib.SMTP()
        server.connect(smtp_host, smtp_port)
        server.ehlo()

        if smtp_use_tls: server.starttls()
        if smtp_need_login: server.login(smtp_user, smtp_password)

        msg = MIMEMultipart()
        msg['From'] = smtp_from
        msg['To'] = COMMASPACE.join(smtp_tolist)
        msg['Subject'] = doorpi.DoorPi().parse_string(smtp_subject)
        msg.attach(MIMEText(doorpi.DoorPi().parse_string(smtp_text), 'html'))
        msg.attach(MIMEText('\nsent by:\n'+doorpi.DoorPi().epilog, 'plain'))

        #add a snapshot
        file = []
        if smtp_snapshot and len(doorpi.DoorPi().config.get('SIP-Phone', 'capture_device', '')) > 0:
            file = createSnapshot()
        if len(file) > 0:
            part = MIMEBase('application',"octet-stream")
            part.set_payload(open(file,"rb").read())
            Encoders.encode_base64(part)
            part.add_header('Content-Disposition', 'attachment; filename="%s"'
                   % os.path.basename(file))
            msg.attach(part)

        server.sendmail(smtp_from, smtp_tolist, msg.as_string())
        server.quit()
    except:
        logger.exception("couldn't send email")
        return False
    return True

def createSnapshot():
    snapshot_file = '/tmp/doorpi.jpg'
    size = doorpi.DoorPi().config.get_string('DoorPi', 'snapshot_size', '1280x720')
    command = "fswebcam --no-banner -r " + size + " " + snapshot_file
    try:
        retcode = subprocess.call(command, shell=True)
        if retcode != 0:
            logger.error('error creating snapshot')
        else:
            logger.info('snapshot created: %s', snapshot_file)
            return snapshot_file

    except OSError as e:
        logger.error('error creating snapshot')
    return ''

def get(parameters):
    parameter_list = parameters.split(',')
    if len(parameter_list) < 3 or len(parameter_list) > 4: return None

    smtp_to = parameter_list[0]
    smtp_subject = parameter_list[1]
    smtp_text = parameter_list[2]
    if (len(parameter_list) == 4):
        smtp_snapshot = parameter_list[3]
    else:
        smtp_snapshot = False
    
    return MailtoAction(fire_action_mail,
                     smtp_to = smtp_to,
                     smtp_subject = smtp_subject,
                     smtp_text = smtp_text,
                     smtp_snapshot = smtp_snapshot)


class MailtoAction(SingleAction):
    pass
